@@ -14,6 +14,7 @@
 #    limitations under the License.
 
 from email import header
+from email.mime import multipart
 from email.mime import text
 import json
 import smtplib
@@ -42,6 +43,8 @@ class EchoAction(actions.Action):
     """
 
     def __init__(self, output):
+        super(EchoAction, self).__init__()
+
         self.output = output
 
     def run(self, context):
@@ -59,9 +62,6 @@ class NoOpAction(actions.Action):
     This action does nothing. It can be mostly useful for testing and
     debugging purposes.
     """
-    def __init__(self):
-        pass
-
     def run(self, context):
         LOG.info('Running no-op action')
 
@@ -80,25 +80,38 @@ class AsyncNoOpAction(NoOpAction):
 class FailAction(actions.Action):
     """'Always fail' action.
 
-    This action just always throws an instance of ActionException.
+    If you pass the `error_data` parameter, this action will be failed and
+    return this data as error data. Otherwise, the action just throws an
+    instance of ActionException.
+
     This behavior is useful in a number of cases, especially if we need to
     test a scenario where some of workflow tasks fail.
+
+    :param error_data: Action will be failed with this data
     """
 
-    def __init__(self):
-        pass
+    def __init__(self, error_data=None):
+        super(FailAction, self).__init__()
+
+        self.error_data = error_data
 
     def run(self, context):
         LOG.info('Running fail action.')
 
+        if self.error_data:
+            return actions.Result(error=self.error_data)
+
         raise exc.ActionException('Fail action expected exception.')
 
     def test(self, context):
+        if self.error_data:
+            return actions.Result(error=self.error_data)
+
         raise exc.ActionException('Fail action expected exception.')
 
 
 class HTTPAction(actions.Action):
-    """Constructs an HTTP action.
+    """HTTP action.
 
     :param url: URL for the new HTTP request.
     :param method: (optional, 'GET' by default) method for the new HTTP
@@ -135,6 +148,7 @@ class HTTPAction(actions.Action):
                  allow_redirects=None,
                  proxies=None,
                  verify=None):
+        super(HTTPAction, self).__init__()
 
         if auth and len(auth.split(':')) == 2:
             self.auth = (auth.split(':')[0], auth.split(':')[1])
@@ -177,6 +191,12 @@ class HTTPAction(actions.Action):
         )
 
         try:
+            url_data = six.moves.urllib.parse.urlsplit(self.url)
+            if 'https' == url_data.scheme:
+                action_verify = self.verify
+            else:
+                action_verify = None
+
             resp = requests.request(
                 self.method,
                 self.url,
@@ -188,9 +208,13 @@ class HTTPAction(actions.Action):
                 timeout=self.timeout,
                 allow_redirects=self.allow_redirects,
                 proxies=self.proxies,
-                verify=self.verify
+                verify=action_verify
             )
         except Exception as e:
+            LOG.exception(
+                "Failed to send HTTP request for action execution: %s",
+                context.execution.action_execution_id
+            )
             raise exc.ActionException("Failed to send HTTP request: %s" % e)
 
         LOG.info(
@@ -199,11 +223,6 @@ class HTTPAction(actions.Action):
             resp.content
         )
 
-        # TODO(akuznetsova): Need to refactor Mistral serialiser and
-        # deserializer to have an ability to pass needed encoding and work
-        # with it. Now it can process only default 'utf-8' encoding.
-        # Appropriate bug #1676411 was created.
-
         # Represent important resp data as a dictionary.
         try:
             content = resp.json(encoding=resp.encoding)
@@ -244,12 +263,12 @@ class MistralHTTPAction(HTTPAction):
         self.headers.update({
             'Mistral-Workflow-Name': exec_ctx.workflow_name,
             'Mistral-Workflow-Execution-Id': exec_ctx.workflow_execution_id,
-            'Mistral-Task-Id': exec_ctx.task_id,
+            'Mistral-Task-Id': exec_ctx.task_execution_id,
             'Mistral-Action-Execution-Id': exec_ctx.action_execution_id,
             'Mistral-Callback-URL': exec_ctx.callback_url,
         })
 
-        super(MistralHTTPAction, self).run(context)
+        return super(MistralHTTPAction, self).run(context)
 
     def is_sync(self):
         return False
@@ -259,14 +278,19 @@ class MistralHTTPAction(HTTPAction):
 
 
 class SendEmailAction(actions.Action):
-    def __init__(self, from_addr, to_addrs, smtp_server,
-                 smtp_password=None, subject=None, body=None):
+    def __init__(self, from_addr, to_addrs, smtp_server, cc_addrs=None,
+                 bcc_addrs=None, smtp_password=None, subject=None, body=None,
+                 html_body=None):
+        super(SendEmailAction, self).__init__()
         # TODO(dzimine): validate parameters
 
         # Task invocation parameters.
         self.to = to_addrs
+        self.cc = cc_addrs or []
+        self.bcc = bcc_addrs or []
         self.subject = subject or "<No subject>"
         self.body = body or "<No body>"
+        self.html_body = html_body
 
         # Action provider settings.
         self.smtp_server = smtp_server
@@ -276,19 +300,35 @@ class SendEmailAction(actions.Action):
     def run(self, context):
         LOG.info(
             "Sending email message "
-            "[from=%s, to=%s, subject=%s, using smtp=%s, body=%s...]",
+            "[from=%s, to=%s, cc=%s, bcc=%s, subject=%s, using smtp=%s, "
+            "body=%s...]",
             self.sender,
             self.to,
+            self.cc,
+            self.bcc,
             self.subject,
             self.smtp_server,
             self.body[:128]
         )
-
-        message = text.MIMEText(self.body, _charset='utf-8')
+        if not self.html_body:
+            message = text.MIMEText(self.body, _charset='utf-8')
+        else:
+            message = multipart.MIMEMultipart('alternative')
+            message.attach(text.MIMEText(self.body,
+                                         'plain',
+                                         _charset='utf-8'))
+            message.attach(text.MIMEText(self.html_body,
+                                         'html',
+                                         _charset='utf-8'))
         message['Subject'] = header.Header(self.subject, 'utf-8')
         message['From'] = self.sender
         message['To'] = ', '.join(self.to)
 
+        if self.cc:
+            message['cc'] = ', '.join(self.cc)
+
+        rcpt = self.cc + self.bcc + self.to
+
         try:
             s = smtplib.SMTP(self.smtp_server)
 
@@ -300,7 +340,7 @@ class SendEmailAction(actions.Action):
                 s.login(self.sender, self.password)
 
             s.sendmail(from_addr=self.sender,
-                       to_addrs=self.to,
+                       to_addrs=rcpt,
                        msg=message.as_string())
         except (smtplib.SMTPException, IOError) as e:
             raise exc.ActionException("Failed to send an email message: %s"
@@ -311,9 +351,12 @@ class SendEmailAction(actions.Action):
         # to return a result.
         LOG.info(
             "Sending email message "
-            "[from=%s, to=%s, subject=%s, using smtp=%s, body=%s...]",
+            "[from=%s, to=%s, cc=%s, bcc=%s, subject=%s, using smtp=%s, "
+            "body=%s...]",
             self.sender,
             self.to,
+            self.cc,
+            self.bcc,
             self.subject,
             self.smtp_server,
             self.body[:128]
@@ -333,7 +376,9 @@ class SSHAction(actions.Action):
         return ssh_utils.execute_command
 
     def __init__(self, cmd, host, username,
-                 password=None, private_key_filename=None):
+                 password="", private_key_filename=None):
+        super(SSHAction, self).__init__()
+
         self.cmd = cmd
         self.host = host
         self.username = username
@@ -352,8 +397,11 @@ class SSHAction(actions.Action):
         def raise_exc(parent_exc=None):
             message = ("Failed to execute ssh cmd "
                        "'%s' on %s" % (self.cmd, self.host))
+            # We suppress the actual parent error messages in favor of
+            # more generic ones as we might be leaking information to the CLI
             if parent_exc:
-                message += "\nException: %s" % str(parent_exc)
+                # The full error message needs to be logged regardless
+                LOG.exception(message + " Exception: %s", str(parent_exc))
             raise exc.ActionException(message)
 
         try:
@@ -380,8 +428,7 @@ class SSHAction(actions.Action):
             return raise_exc(parent_exc=e)
 
     def test(self, context):
-        # TODO(rakhmerov): Implement.
-        return None
+        return json.dumps(self.params)
 
 
 class SSHProxiedAction(SSHAction):
@@ -422,6 +469,7 @@ class JavaScriptAction(actions.Action):
 
         Not the usual mistral context. That is passed during the run method
         """
+        super(JavaScriptAction, self).__init__()
 
         self.script = script
         self.js_context = context
@@ -448,6 +496,8 @@ class SleepAction(actions.Action):
     for testing and debugging purposes.
     """
     def __init__(self, seconds=1):
+        super(SleepAction, self).__init__()
+
         try:
             self._seconds = int(seconds)
             self._seconds = 0 if self._seconds < 0 else self._seconds
@@ -471,6 +521,8 @@ class TestDictAction(actions.Action):
     """Generates test dict."""
 
     def __init__(self, size=0, key_prefix='', val=''):
+        super(TestDictAction, self).__init__()
+
         self.size = size
         self.key_prefix = key_prefix
         self.val = val
