@@ -1,4 +1,5 @@
 # Copyright 2013 - Mirantis, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -17,16 +18,30 @@ from wsme import types as wtypes
 
 from mistral.api.controllers import resource
 from mistral.api.controllers.v2 import types
+from mistral import exceptions as exc
+from mistral import utils
 from mistral.workflow import states
 
 SCOPE_TYPES = wtypes.Enum(str, 'private', 'public')
 
 
-class Workbook(resource.Resource):
+class ScopedResource(object):
+    """Utilities for scoped resources"""
+    @classmethod
+    def validate_scope(cls, scope):
+        if scope not in SCOPE_TYPES.values:
+            raise exc.InvalidModelException(
+                "Scope must be one of the following: %s; actual: "
+                "%s" % (SCOPE_TYPES.values, scope)
+            )
+
+
+class Workbook(resource.Resource, ScopedResource):
     """Workbook resource."""
 
     id = wtypes.text
     name = wtypes.text
+    namespace = wtypes.text
 
     definition = wtypes.text
     "workbook definition in Mistral v2 DSL"
@@ -48,7 +63,8 @@ class Workbook(resource.Resource):
                    scope='private',
                    project_id='a7eb669e9819420ea4bd1453e672c0a7',
                    created_at='1970-01-01T00:00:00.000000',
-                   updated_at='1970-01-01T00:00:00.000000')
+                   updated_at='1970-01-01T00:00:00.000000',
+                   namespace='')
 
 
 class Workbooks(resource.ResourceList):
@@ -66,7 +82,7 @@ class Workbooks(resource.ResourceList):
         return cls(workbooks=[Workbook.sample()])
 
 
-class Workflow(resource.Resource):
+class Workflow(resource.Resource, ScopedResource):
     """Workflow resource."""
 
     id = wtypes.text
@@ -128,6 +144,22 @@ class Workflow(resource.Resource):
 
         return cls._set_input(obj, db_model.spec)
 
+    @classmethod
+    def from_tuples(cls, tuple_iterator):
+        obj = cls()
+        spec = None
+        for col_name, col_val in tuple_iterator:
+            if hasattr(obj, col_name):
+                # Convert all datetime values to strings.
+                setattr(obj, col_name, utils.datetime_to_str(col_val))
+            if col_name == 'spec':
+                spec = col_val
+
+        if spec:
+            obj = cls._set_input(obj, spec)
+
+        return obj
+
 
 class Workflows(resource.ResourceList):
     """A collection of workflows."""
@@ -151,7 +183,7 @@ class Workflows(resource.ResourceList):
         return workflows_sample
 
 
-class Action(resource.Resource):
+class Action(resource.Resource, ScopedResource):
     """Action resource.
 
     NOTE: *name* is immutable. Note that name and description get inferred
@@ -284,18 +316,39 @@ class Execution(resource.Resource):
 
     @classmethod
     def sample(cls):
-        return cls(id='123e4567-e89b-12d3-a456-426655440000',
-                   workflow_name='flow',
-                   workflow_namespace='some_namespace',
-                   workflow_id='123e4567-e89b-12d3-a456-426655441111',
-                   description='this is the first execution.',
-                   project_id='40a908dbddfe48ad80a87fb30fa70a03',
-                   state='SUCCESS',
-                   input={},
-                   output={},
-                   params={'env': {'k1': 'abc', 'k2': 123}},
-                   created_at='1970-01-01T00:00:00.000000',
-                   updated_at='1970-01-01T00:00:00.000000')
+        return cls(
+            id='123e4567-e89b-12d3-a456-426655440000',
+            workflow_name='flow',
+            workflow_namespace='some_namespace',
+            workflow_id='123e4567-e89b-12d3-a456-426655441111',
+            description='this is the first execution.',
+            project_id='40a908dbddfe48ad80a87fb30fa70a03',
+            state='SUCCESS',
+            input={},
+            output={},
+            params={
+                'env': {'k1': 'abc', 'k2': 123},
+                'notify': [
+                    {
+                        'type': 'webhook',
+                        'url': 'http://endpoint/of/webhook',
+                        'headers': {
+                            'Content-Type': 'application/json',
+                            'X-Auth-Token': '123456789'
+                        }
+                    },
+                    {
+                        'type': 'queue',
+                        'topic': 'failover_queue',
+                        'backend': 'rabbitmq',
+                        'host': '127.0.0.1',
+                        'port': 5432
+                    }
+                ]
+            },
+            created_at='1970-01-01T00:00:00.000000',
+            updated_at='1970-01-01T00:00:00.000000'
+        )
 
 
 class Executions(resource.ResourceList):
@@ -351,6 +404,9 @@ class Task(resource.Resource):
     created_at = wtypes.text
     updated_at = wtypes.text
 
+    started_at = wtypes.text
+    finished_at = wtypes.text
+
     # Add this param to make Mistral API work with WSME 0.8.0 or higher version
     reset = wsme.wsattr(bool, mandatory=True)
 
@@ -665,3 +721,158 @@ class EventTriggers(resource.ResourceList):
                                 "marker=123e4567-e89b-12d3-a456-426655440000")
 
         return triggers_sample
+
+
+class BaseExecutionReportEntry(resource.Resource):
+    """Execution report entry resource."""
+
+    id = wtypes.text
+    name = wtypes.text
+    created_at = wtypes.text
+    updated_at = wtypes.text
+    state = wtypes.text
+    state_info = wtypes.text
+
+    @classmethod
+    def sample(cls):
+        # TODO(rakhmerov): complete
+
+        return cls(
+            id='123e4567-e89b-12d3-a456-426655441414',
+            created_at='2019-01-30T00:00:00.000000',
+            updated_at='2019-01-30T00:00:00.000000',
+            state=states.SUCCESS
+        )
+
+
+class ActionExecutionReportEntry(BaseExecutionReportEntry):
+    """Action execution report entry resource."""
+
+    accepted = bool
+    last_heartbeat = wtypes.text
+
+    @classmethod
+    def sample(cls):
+        sample = super(ActionExecutionReportEntry, cls).sample()
+
+        sample.accepted = True
+        sample.last_heartbeat = '2019-01-30T00:00:00.000000'
+
+        return sample
+
+
+class WorkflowExecutionReportEntry(BaseExecutionReportEntry):
+    """Workflow execution report entry resource."""
+
+    # NOTE(rakhmerov): task_executions has to be declared below
+    # after we declare a class for task execution entry resource.
+
+    @classmethod
+    def sample(cls):
+        sample = super(WorkflowExecutionReportEntry, cls).sample()
+
+        # We can't define a non-empty list task executions here because
+        # the needed class is not defined yet. Since this is just a sample
+        # we can sacrifice it.
+        sample.task_executions = []
+
+        return sample
+
+
+class TaskExecutionReportEntry(BaseExecutionReportEntry):
+    """Task execution report entity resource."""
+
+    action_executions = [ActionExecutionReportEntry]
+    workflow_executions = [WorkflowExecutionReportEntry]
+
+    @classmethod
+    def sample(cls):
+        sample = super(TaskExecutionReportEntry, cls).sample()
+
+        sample.action_executions = [ActionExecutionReportEntry.sample()]
+        sample.workflow_executions = []
+
+        return sample
+
+
+# We have to declare this field later because of the dynamic binding.
+# It can't be within WorkflowExecutionReportEntry before
+# TaskExecutionReportEntry is declared.
+WorkflowExecutionReportEntry.task_executions = [TaskExecutionReportEntry]
+wtypes.registry.reregister(WorkflowExecutionReportEntry)
+
+
+class ExecutionReportStatistics(resource.Resource):
+    """Execution report statistics.
+
+    TODO(rakhmerov): There's much more we can add here. For example,
+    information about action, average (and also min and max) task execution
+    run time etc.
+    """
+
+    total_tasks_count = wtypes.IntegerType(minimum=0)
+    running_tasks_count = wtypes.IntegerType(minimum=0)
+    success_tasks_count = wtypes.IntegerType(minimum=0)
+    error_tasks_count = wtypes.IntegerType(minimum=0)
+    idle_tasks_count = wtypes.IntegerType(minimum=0)
+    paused_tasks_count = wtypes.IntegerType(minimum=0)
+
+    def __init__(self, **kw):
+        self.total_tasks_count = 0
+        self.running_tasks_count = 0
+        self.success_tasks_count = 0
+        self.error_tasks_count = 0
+        self.idle_tasks_count = 0
+        self.paused_tasks_count = 0
+
+        super(ExecutionReportStatistics, self).__init__(**kw)
+
+    def increment_running(self):
+        self.running_tasks_count += 1
+        self.total_tasks_count += 1
+
+    def increment_success(self):
+        self.success_tasks_count += 1
+        self.total_tasks_count += 1
+
+    def increment_error(self):
+        self.error_tasks_count += 1
+        self.total_tasks_count += 1
+
+    def increment_idle(self):
+        self.idle_tasks_count += 1
+        self.total_tasks_count += 1
+
+    def increment_paused(self):
+        self.paused_tasks_count += 1
+        self.total_tasks_count += 1
+
+    @classmethod
+    def sample(cls):
+        return cls(
+            total_tasks_count=10,
+            running_tasks_count=3,
+            success_tasks_count=5,
+            error_tasks_count=2,
+            idle_tasks_count=0,
+            paused_tasks_count=0
+        )
+
+
+class ExecutionReport(resource.Resource):
+    """Execution report resource."""
+
+    statistics = ExecutionReportStatistics
+    """General statistics about the workflow execution hierarchy."""
+
+    root_workflow_execution = WorkflowExecutionReportEntry
+    """Root entry of the report associated with a workflow execution."""
+
+    @classmethod
+    def sample(cls):
+        sample = cls()
+
+        sample.statistics = ExecutionReportStatistics.sample()
+        sample.root_workflow_execution = WorkflowExecutionReportEntry.sample()
+
+        return sample
