@@ -1,5 +1,6 @@
 #!/usr/bin/env python
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -46,6 +47,7 @@ from mistral import config
 from mistral.engine import engine_server
 from mistral.event_engine import event_engine_server
 from mistral.executors import executor_server
+from mistral.notifiers import notification_server
 from mistral.rpc import base as rpc
 from mistral import version
 
@@ -54,13 +56,16 @@ CONF = cfg.CONF
 SERVER_THREAD_MANAGER = None
 SERVER_PROCESS_MANAGER = None
 
+LOG = logging.getLogger(__name__)
+
 
 def launch_thread(server, workers=1):
     try:
         global SERVER_THREAD_MANAGER
 
         if not SERVER_THREAD_MANAGER:
-            SERVER_THREAD_MANAGER = service.ServiceLauncher(CONF)
+            SERVER_THREAD_MANAGER = service.ServiceLauncher(
+                CONF, restart_method='mutate')
 
         SERVER_THREAD_MANAGER.launch_service(server, workers=workers)
     except Exception as e:
@@ -73,7 +78,8 @@ def launch_process(server, workers=1):
         global SERVER_PROCESS_MANAGER
 
         if not SERVER_PROCESS_MANAGER:
-            SERVER_PROCESS_MANAGER = service.ProcessLauncher(CONF)
+            SERVER_PROCESS_MANAGER = service.ProcessLauncher(
+                CONF, restart_method='mutate')
 
         SERVER_PROCESS_MANAGER.launch_service(server, workers=workers)
     except Exception as e:
@@ -93,8 +99,13 @@ def launch_event_engine():
     launch_thread(event_engine_server.get_oslo_service())
 
 
+def launch_notifier():
+    launch_thread(notification_server.get_oslo_service())
+
+
 def launch_api():
     server = api_service.WSGIService('mistral_api')
+
     launch_process(server, workers=server.workers)
 
 
@@ -118,7 +129,8 @@ LAUNCH_OPTIONS = {
     'api': launch_api,
     'engine': launch_engine,
     'executor': launch_executor,
-    'event-engine': launch_event_engine
+    'event-engine': launch_event_engine,
+    'notifier': launch_notifier
 }
 
 
@@ -131,14 +143,14 @@ MISTRAL_TITLE = """
 ||      || || __//   ||_// ||      \\\__// \\\_ ||
 
 Mistral Workflow Service, version %s
-""" % version.version_string()
+""" % version.version_string
 
 
 def print_server_info():
     print(MISTRAL_TITLE)
 
-    comp_str = ("[%s]" % ','.join(LAUNCH_OPTIONS)
-                if cfg.CONF.server == ['all'] else cfg.CONF.server)
+    comp_str = ("[%s]" % ','.join(LAUNCH_OPTIONS
+                if cfg.CONF.server == ['all'] else cfg.CONF.server))
 
     print('Launching server components %s...' % comp_str)
 
@@ -168,12 +180,28 @@ def get_properly_ordered_parameters():
     return args
 
 
+def override_keystone_options():
+    # TODO(wxy): This function is used for keeping backward compatibility.
+    # Remove it in Stein.
+    auth_opts = CONF['keystone_authtoken']
+    for opt, value in auth_opts.items():
+        if opt in CONF['keystone']:
+            default_value = auth_opts._group._opts[opt]['opt'].default
+            if default_value != value != CONF['keystone'][opt]:
+                LOG.warning("The config option '%s' in section "
+                            "[keystone_authtoken] has the same copy in "
+                            "[keystone]. Please add the same option to the "
+                            "[keystone] section to keep using it.", opt)
+                CONF.set_override(opt, value, group='keystone')
+
+
 def main():
     try:
         config.parse_args(get_properly_ordered_parameters())
         print_server_info()
 
         logging.setup(CONF, 'Mistral')
+        override_keystone_options()
 
         # Please refer to the oslo.messaging documentation for transport
         # configuration. The default transport for oslo.messaging is
