@@ -78,7 +78,7 @@ api_opts = [
     cfg.BoolOpt(
         'enable_ssl_api',
         default=False,
-        help=_('Enable the integrated stand-alone API to service requests'
+        help=_('Enable the integrated stand-alone API to service requests '
                'via HTTPS instead of HTTP.')
     ),
     cfg.IntOpt(
@@ -263,6 +263,34 @@ scheduler_opts = [
             'If this property equals None then there is no '
             'restriction on selection.'
         )
+    ),
+    cfg.FloatOpt(
+        'captured_job_timeout',
+        default=30,
+        min=1,
+        help=(
+            'Defines how soon (in seconds) a scheduled job captured for '
+            'processing becomes eligible for capturing by other schedulers '
+            'again. This option is needed to prevent situations when a '
+            'scheduler instance captured a job and failed while processing '
+            'and so this job can never be processed again because it is '
+            'marked as captured.'
+        )
+    ),
+    cfg.FloatOpt(
+        'pickup_job_after',
+        default=60,
+        min=1,
+        help='Time period given to a scheduler to process a scheduled job '
+             'locally before it becomes eligible for processing by other '
+             'scheduler instances.'
+             'For example, a job needs to run at 12:00:00. When a scheduler '
+             'starts processing it it has 60 seconds (or other configured '
+             'value) to complete the job. If the scheduler did not complete '
+             'the job within this period it most likely means that the '
+             'scheduler process crashed. In this case another scheduler '
+             'instance will pick it up from the Job Store, but not earlier '
+             'than 12:01:00 and try to process it.'
     )
 ]
 
@@ -532,23 +560,6 @@ yaql_opts = [
         help=_('The memory usage quota (in bytes) for all data produced by '
                'the expression (or any part of it). -1 means no limitation.')
     ),
-    cfg.BoolOpt(
-        'convert_input_data',
-        default=True,
-        help=_('Enables input data conversion for YAQL expressions. If set '
-               'to True then YAQL will convert mutable data structures '
-               '(lists, dicts, sets) into their immutable versions. That '
-               'will allow them to work with some constructs that require '
-               'hashable types even if elements are not hashable. For '
-               'example, it will be possible to put dicts into a set. '
-               'Although it conflicts with the base principles of such '
-               'collections (e.g. we cannot put a non-hashable type into '
-               'a set just because otherwise it will not work correctly) the '
-               'YAQL library itself allows this. '
-               'Disabling input data conversion may give significant '
-               'performance boost if the input data for an expression is '
-               'large.')
-    ),
     cfg.BoolOpt(
         'convert_tuples_to_lists',
         default=True,
