@@ -255,6 +255,8 @@ class TaskExecution(Execution):
     action_spec = sa.Column(st.JsonLongDictType())
     unique_key = sa.Column(sa.String(255), nullable=True)
     type = sa.Column(sa.String(10))
+    started_at = sa.Column(sa.DateTime, nullable=True)
+    finished_at = sa.Column(sa.DateTime, nullable=True)
 
     # Whether the task is fully processed (publishing and calculating commands
     # after it). It allows to simplify workflow controller implementations
@@ -273,6 +275,14 @@ class TaskExecution(Execution):
             else self.workflow_executions
         )
 
+    def to_dict(self):
+        d = super(TaskExecution, self).to_dict()
+
+        utils.datetime_to_str_in_dict(d, 'started_at')
+        utils.datetime_to_str_in_dict(d, 'finished_at')
+
+        return d
+
 
 for cls in utils.iter_subclasses(Execution):
     event.listen(
@@ -343,6 +353,13 @@ WorkflowExecution.root_execution_id = sa.Column(
     nullable=True
 )
 
+WorkflowExecution.root_execution = relationship(
+    WorkflowExecution,
+    remote_side=WorkflowExecution.id,
+    lazy='select'
+)
+
+
 # Many-to-one for 'TaskExecution' and 'WorkflowExecution'.
 
 TaskExecution.workflow_execution_id = sa.Column(
@@ -395,6 +412,39 @@ sa.Index(
 )
 
 
+class ScheduledJob(mb.MistralModelBase):
+    """Contains info about scheduled jobs."""
+
+    __tablename__ = 'scheduled_jobs_v2'
+
+    id = mb.id_column()
+
+    run_after = sa.Column(sa.Integer)
+
+    # The full name of the factory function that returns/builds a Python
+    # (target) object whose method should be called. Optional.
+    target_factory_func_name = sa.Column(sa.String(200), nullable=True)
+
+    # May take two different forms:
+    # 1. Full path of a target function that should be called. For example,
+    #  "mistral.utils.random_sleep".
+    # 2. Name of a method to call on a target object, if
+    #  "target_factory_func_name" is specified.
+    func_name = sa.Column(sa.String(80), nullable=False)
+
+    func_args = sa.Column(st.JsonDictType())
+    func_arg_serializers = sa.Column(st.JsonDictType())
+    auth_ctx = sa.Column(st.JsonDictType())
+    execute_at = sa.Column(sa.DateTime, nullable=False)
+    captured_at = sa.Column(sa.DateTime, nullable=True)
+
+
+sa.Index(
+    '%s_execution_time' % ScheduledJob.__tablename__,
+    ScheduledJob.execute_at
+)
+
+
 class Environment(mb.MistralSecureModelBase):
     """Contains environment variables for workflow execution."""
 
