@@ -1,5 +1,6 @@
 # Copyright 2016 - Nokia Networks.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -20,7 +21,7 @@ from osprofiler import profiler
 import six
 
 from mistral.db.v2 import api as db_api
-from mistral.engine import action_queue
+from mistral.engine import post_tx_queue
 from mistral.engine import utils as engine_utils
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions as exc
@@ -32,6 +33,7 @@ from mistral.services import security
 from mistral import utils
 from mistral.utils import wf_trace
 from mistral.workflow import data_flow
+from mistral.workflow import lookup_utils
 from mistral.workflow import states
 from mistral_lib import actions as ml_actions
 
@@ -148,7 +150,7 @@ class Action(object):
         """
         return True
 
-    def _create_action_execution(self, input_dict, runtime_ctx,
+    def _create_action_execution(self, input_dict, runtime_ctx, is_sync,
                                  desc='', action_ex_id=None):
         action_ex_id = action_ex_id or utils.generate_unicode_uuid()
 
@@ -159,7 +161,8 @@ class Action(object):
             'state': states.RUNNING,
             'input': input_dict,
             'runtime_context': runtime_ctx,
-            'description': desc
+            'description': desc,
+            'is_sync': is_sync
         }
 
         if self.task_ex:
@@ -208,12 +211,18 @@ class Action(object):
 class PythonAction(Action):
     """Regular Python action."""
 
+    def __init__(self, action_def, action_ex=None, task_ex=None):
+        super(PythonAction, self).__init__(action_def, action_ex, task_ex)
+
+        self._prepared_input = None
+
     @profiler.trace('action-complete', hide_args=True)
     def complete(self, result):
         assert self.action_ex
 
         if states.is_completed(self.action_ex.state):
-            return
+            raise ValueError(
+                "Action {} is already completed".format(self.action_ex.id))
 
         prev_state = self.action_ex.state
 
@@ -234,6 +243,8 @@ class PythonAction(Action):
                  timeout=None):
         assert not self.action_ex
 
+        self.validate_input(input_dict)
+
         # Assign the action execution ID here to minimize database calls.
         # Otherwise, the input property of the action execution DB object needs
         # to be updated with the action execution ID after the action execution
@@ -243,27 +254,39 @@ class PythonAction(Action):
         self._create_action_execution(
             self._prepare_input(input_dict),
             self._prepare_runtime_context(index, safe_rerun),
+            self.is_sync(input_dict),
             desc=desc,
             action_ex_id=action_ex_id
         )
 
         execution_context = self._prepare_execution_context()
 
-        action_queue.schedule_run_action(
-            self.action_ex,
-            self.action_def,
-            target,
-            execution_context,
-            timeout=timeout
-        )
+        # Register an asynchronous command to send the action to
+        # run on an executor outside of the main DB transaction.
+        def _run_action():
+            executor = exe.get_executor(cfg.CONF.executor.type)
+
+            executor.run_action(
+                self.action_ex.id,
+                self.action_def.action_class,
+                self.action_def.attributes or {},
+                self.action_ex.input,
+                self.action_ex.runtime_context.get('safe_rerun', False),
+                execution_context,
+                target=target,
+                timeout=timeout
+            )
+
+        post_tx_queue.register_operation(_run_action)
 
     @profiler.trace('action-run', hide_args=True)
     def run(self, input_dict, target, index=0, desc='', save=True,
             safe_rerun=False, timeout=None):
         assert not self.action_ex
 
-        input_dict = self._prepare_input(input_dict)
-        runtime_ctx = self._prepare_runtime_context(index, safe_rerun)
+        self.validate_input(input_dict)
+
+        prepared_input_dict = self._prepare_input(input_dict)
 
         # Assign the action execution ID here to minimize database calls.
         # Otherwise, the input property of the action execution DB object needs
@@ -273,8 +296,9 @@ class PythonAction(Action):
 
         if save:
             self._create_action_execution(
-                input_dict,
-                runtime_ctx,
+                prepared_input_dict,
+                self._prepare_runtime_context(index, safe_rerun),
+                self.is_sync(input_dict),
                 desc=desc,
                 action_ex_id=action_ex_id
             )
@@ -287,7 +311,7 @@ class PythonAction(Action):
             self.action_ex.id if self.action_ex else None,
             self.action_def.action_class,
             self.action_def.attributes or {},
-            input_dict,
+            prepared_input_dict,
             safe_rerun,
             execution_context,
             target=target,
@@ -298,14 +322,13 @@ class PythonAction(Action):
         return self._prepare_output(result)
 
     def is_sync(self, input_dict):
-        input_dict = self._prepare_input(input_dict)
+        prepared_input_dict = self._prepare_input(input_dict)
 
-        a = a_m.get_action_class(self.action_def.name)(**input_dict)
+        a = a_m.get_action_class(self.action_def.name)(**prepared_input_dict)
 
         return a.is_sync()
 
     def validate_input(self, input_dict):
-
         # NOTE(kong): Don't validate action input if action initialization
         # method contains ** argument.
         if '**' in self.action_def.input:
@@ -321,19 +344,20 @@ class PythonAction(Action):
         )
 
     def _prepare_execution_context(self):
-
         exc_ctx = {}
 
         if self.task_ex:
             wf_ex = self.task_ex.workflow_execution
+
             exc_ctx['workflow_execution_id'] = wf_ex.id
-            exc_ctx['task_id'] = self.task_ex.id
+            exc_ctx['task_execution_id'] = self.task_ex.id
             exc_ctx['workflow_name'] = wf_ex.name
 
         if self.action_ex:
             exc_ctx['action_execution_id'] = self.action_ex.id
-            callback_url = '/v2/action_executions/%s' % self.action_ex.id
-            exc_ctx['callback_url'] = callback_url
+            exc_ctx['callback_url'] = (
+                '/v2/action_executions/%s' % self.action_ex.id
+            )
 
         return exc_ctx
 
@@ -363,15 +387,16 @@ class PythonAction(Action):
 class AdHocAction(PythonAction):
     """Ad-hoc action."""
 
+    @profiler.trace('ad-hoc-action-init', hide_args=True)
     def __init__(self, action_def, action_ex=None, task_ex=None, task_ctx=None,
                  wf_ctx=None):
         self.action_spec = spec_parser.get_action_spec(action_def.spec)
 
-        try:
-            base_action_def = db_api.get_action_definition(
-                self.action_spec.get_base()
-            )
-        except exc.DBEntityNotFoundError:
+        base_action_def = lookup_utils.find_action_definition_by_name(
+            self.action_spec.get_base()
+        )
+
+        if not base_action_def:
             raise exc.InvalidActionException(
                 "Failed to find action [action_name=%s]" %
                 self.action_spec.get_base()
@@ -392,6 +417,7 @@ class AdHocAction(PythonAction):
         self.task_ctx = task_ctx or {}
         self.wf_ctx = wf_ctx or {}
 
+    @profiler.trace('ad-hoc-action-validate-input', hide_args=True)
     def validate_input(self, input_dict):
         expected_input = self.action_spec.get_input()
 
@@ -406,11 +432,16 @@ class AdHocAction(PythonAction):
             self._prepare_input(input_dict)
         )
 
+    @profiler.trace('ad-hoc-action-prepare-input', hide_args=True)
     def _prepare_input(self, input_dict):
+        if self._prepared_input is not None:
+            return self._prepared_input
+
         base_input_dict = input_dict
 
         for action_def in self.adhoc_action_defs:
             action_spec = spec_parser.get_action_spec(action_def.spec)
+
             for k, v in action_spec.get_input().items():
                 if (k not in base_input_dict or
                         base_input_dict[k] is utils.NotDefined):
@@ -437,8 +468,13 @@ class AdHocAction(PythonAction):
             else:
                 base_input_dict = {}
 
-        return super(AdHocAction, self)._prepare_input(base_input_dict)
+        self._prepared_input = super(AdHocAction, self)._prepare_input(
+            base_input_dict
+        )
 
+        return self._prepared_input
+
+    @profiler.trace('ad-hoc-action-prepare-output', hide_args=True)
     def _prepare_output(self, result):
         # In case of error, we don't transform a result.
         if not result.is_error():
@@ -460,6 +496,7 @@ class AdHocAction(PythonAction):
 
         return result
 
+    @profiler.trace('ad-hoc-action-prepare-runtime-context', hide_args=True)
     def _prepare_runtime_context(self, index, safe_rerun):
         ctx = super(AdHocAction, self)._prepare_runtime_context(
             index,
@@ -473,6 +510,7 @@ class AdHocAction(PythonAction):
             {'adhoc_action_name': self.adhoc_action_def.name}
         )
 
+    @profiler.trace('ad-hoc-action-gather-base-actions', hide_args=True)
     def _gather_base_actions(self, action_def, base_action_def):
         """Find all base ad-hoc actions and store them
 
@@ -521,6 +559,7 @@ class WorkflowAction(Action):
 
     def __init__(self, wf_name, **kwargs):
         super(WorkflowAction, self).__init__(None, **kwargs)
+
         self.wf_name = wf_name
 
     @profiler.trace('workflow-action-complete', hide_args=True)
@@ -528,11 +567,13 @@ class WorkflowAction(Action):
         # No-op because in case of workflow result is already processed.
         pass
 
-    @profiler.trace('workflkow-action-schedule', hide_args=True)
+    @profiler.trace('workflow-action-schedule', hide_args=True)
     def schedule(self, input_dict, target, index=0, desc='', safe_rerun=False,
                  timeout=None):
         assert not self.action_ex
 
+        self.validate_input(input_dict)
+
         parent_wf_ex = self.task_ex.workflow_execution
         parent_wf_spec = spec_parser.get_workflow_spec_by_execution_id(
             parent_wf_ex.id
@@ -551,7 +592,7 @@ class WorkflowAction(Action):
         )
 
         # If the parent has a root_execution_id, it must be a sub-workflow. So
-        # we should propogate that ID down. Otherwise the parent must be the
+        # we should propagate that ID down. Otherwise the parent must be the
         # root execution and we should use the parents ID.
         root_execution_id = parent_wf_ex.root_execution_id or parent_wf_ex.id
 
@@ -562,6 +603,9 @@ class WorkflowAction(Action):
             'namespace': parent_wf_ex.params['namespace']
         }
 
+        if 'notify' in parent_wf_ex.params:
+            wf_params['notify'] = parent_wf_ex.params['notify']
+
         for k, v in list(input_dict.items()):
             if k not in wf_spec.get_input():
                 wf_params[k] = v
@@ -612,10 +656,14 @@ def resolve_action_definition(action_spec_name, wf_name=None,
 
         action_full_name = "%s.%s" % (wb_name, action_spec_name)
 
-        action_db = db_api.load_action_definition(action_full_name)
+        action_db = lookup_utils.find_action_definition_by_name(
+            action_full_name
+        )
 
     if not action_db:
-        action_db = db_api.load_action_definition(action_spec_name)
+        action_db = lookup_utils.find_action_definition_by_name(
+            action_spec_name
+        )
 
     if not action_db:
         raise exc.InvalidActionException(
