@@ -1,6 +1,7 @@
 # Copyright 2013 - Mirantis, Inc.
 # Copyright 2015 - StackStorm, Inc.
 # Copyright 2016 - Brocade Communications Systems, Inc.
+# Copyright 2018 - Extreme Networks, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License");
 #    you may not use this file except in compliance with the License.
@@ -14,14 +15,16 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+from oslo_config import cfg
+from oslo_log import log as logging
 from osprofiler import profiler
 
 from mistral.db import utils as db_utils
 from mistral.db.v2 import api as db_api
 from mistral.db.v2.sqlalchemy import models as db_models
 from mistral.engine import action_handler
-from mistral.engine import action_queue
 from mistral.engine import base
+from mistral.engine import post_tx_queue
 from mistral.engine import workflow_handler as wf_handler
 from mistral import exceptions
 from mistral import utils as u
@@ -32,16 +35,24 @@ from mistral.workflow import states
 # options required at top level of this  __init__.py are not imported before
 # the submodules are referenced.
 
+LOG = logging.getLogger(__name__)
+
 
 class DefaultEngine(base.Engine):
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     @profiler.trace('engine-start-workflow', hide_args=True)
     def start_workflow(self, wf_identifier, wf_namespace='', wf_ex_id=None,
                        wf_input=None, description='', **params):
         if wf_namespace:
             params['namespace'] = wf_namespace
 
+        if cfg.CONF.notifier.notify:
+            if 'notify' not in params or not params['notify']:
+                params['notify'] = []
+
+            params['notify'].extend(cfg.CONF.notifier.notify)
+
         try:
             with db_api.transaction():
                 wf_ex = wf_handler.start_workflow(
@@ -53,6 +64,9 @@ class DefaultEngine(base.Engine):
                     params
                 )
 
+                # Checking a case when all tasks are completed immediately.
+                wf_handler.check_and_complete(wf_ex.id)
+
                 return wf_ex.get_clone()
 
         except exceptions.DBDuplicateEntryError:
@@ -65,7 +79,7 @@ class DefaultEngine(base.Engine):
                 return wf_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     def start_action(self, action_name, action_input,
                      description=None, **params):
         with db_api.transaction():
@@ -89,8 +103,12 @@ class DefaultEngine(base.Engine):
 
                 return action.action_ex.get_clone()
 
-            output = action.run(action_input, target, save=False,
-                                timeout=timeout)
+            output = action.run(
+                action_input,
+                target,
+                save=False,
+                timeout=timeout
+            )
 
             state = states.SUCCESS if output.is_success() else states.ERROR
 
@@ -114,12 +132,13 @@ class DefaultEngine(base.Engine):
                 'input': action_input,
                 'output': output.to_dict(),
                 'state': state,
+                'is_sync': is_action_sync
             }
 
             return db_api.create_action_execution(values)
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     @profiler.trace('engine-on-action-complete', hide_args=True)
     def on_action_complete(self, action_ex_id, result, wf_action=False,
                            async_=False):
@@ -134,7 +153,7 @@ class DefaultEngine(base.Engine):
             return action_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     @profiler.trace('engine-on-action-update', hide_args=True)
     def on_action_update(self, action_ex_id, state, wf_action=False,
                          async_=False):
@@ -149,7 +168,7 @@ class DefaultEngine(base.Engine):
             return action_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     def pause_workflow(self, wf_ex_id):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -159,7 +178,7 @@ class DefaultEngine(base.Engine):
             return wf_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     def rerun_workflow(self, task_ex_id, reset=True, env=None):
         with db_api.transaction():
             task_ex = db_api.get_task_execution(task_ex_id)
@@ -171,7 +190,7 @@ class DefaultEngine(base.Engine):
             return wf_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     def resume_workflow(self, wf_ex_id, env=None):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -181,7 +200,7 @@ class DefaultEngine(base.Engine):
             return wf_ex.get_clone()
 
     @db_utils.retry_on_db_error
-    @action_queue.process
+    @post_tx_queue.run
     def stop_workflow(self, wf_ex_id, state, message=None):
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex_id)
@@ -193,3 +212,23 @@ class DefaultEngine(base.Engine):
     def rollback_workflow(self, wf_ex_id):
         # TODO(rakhmerov): Implement.
         raise NotImplementedError
+
+    @db_utils.retry_on_db_error
+    @post_tx_queue.run
+    def report_running_actions(self, action_ex_ids):
+        with db_api.transaction():
+            now = u.utc_now_sec()
+
+            for exec_id in action_ex_ids:
+                try:
+                    db_api.update_action_execution(
+                        exec_id,
+                        {"last_heartbeat": now},
+                        insecure=True
+                    )
+                except exceptions.DBEntityNotFoundError:
+                    LOG.debug("Action execution heartbeat update failed. {}"
+                              .format(exec_id), exc_info=True)
+                    # Ignore this error and continue with the
+                    # remaining ids.
+                    pass
