@@ -19,6 +19,7 @@ from mistral.db.v2 import api as db_api
 from mistral.engine import default_engine
 from mistral.rpc import base as rpc
 from mistral.service import base as service_base
+from mistral.services import action_execution_checker
 from mistral.services import expiration_policy
 from mistral.services import scheduler
 from mistral import utils
@@ -51,6 +52,8 @@ class EngineServer(service_base.MistralService):
         self._scheduler = scheduler.start()
         self._expiration_policy_tg = expiration_policy.setup()
 
+        action_execution_checker.start()
+
         if self._setup_profiler:
             profiler_utils.setup('mistral-engine', cfg.CONF.engine.host)
 
@@ -59,16 +62,15 @@ class EngineServer(service_base.MistralService):
         self._rpc_server = rpc.get_rpc_server_driver()(cfg.CONF.engine)
         self._rpc_server.register_endpoint(self)
 
-        # Note(ddeja): Engine needs to be run in default (blocking) mode
-        # since using another mode may lead to a deadlock.
-        # See https://review.openstack.org/#/c/356343 for more info.
-        self._rpc_server.run(executor='blocking')
+        self._rpc_server.run(executor=cfg.CONF.oslo_rpc_executor)
 
         self._notify_started('Engine server started.')
 
     def stop(self, graceful=False):
         super(EngineServer, self).stop(graceful)
 
+        action_execution_checker.stop(graceful)
+
         if self._scheduler:
             scheduler.stop_scheduler(self._scheduler, graceful)
 
@@ -84,7 +86,7 @@ class EngineServer(service_base.MistralService):
 
         :param rpc_ctx: RPC request context.
         :param wf_identifier: Workflow definition identifier.
-        :param wf_namespace: Workflow definition identifier.
+        :param wf_namespace: Workflow namespace.
         :param wf_input: Workflow input.
         :param wf_ex_id: Workflow execution id. If passed, it will be set
             in the new execution object.
@@ -258,6 +260,19 @@ class EngineServer(service_base.MistralService):
 
         return self.engine.rollback_workflow(wf_ex_id)
 
+    def report_running_actions(self, rpc_ctx, action_ex_ids):
+        """Receives calls over RPC to receive action execution heartbeats.
+
+        :param rpc_ctx: RPC request context.
+        :param action_ex_ids: Action execution ids.
+        """
+        LOG.info(
+            "Received RPC request 'report_running_actions'[action_ex_ids=%s]",
+            action_ex_ids
+        )
+
+        return self.engine.report_running_actions(action_ex_ids)
+
 
 def get_oslo_service(setup_profiler=True):
     return EngineServer(
