@@ -381,6 +381,11 @@ class ExecutorClient(exe.Executor):
         rpc_client_method = (self._client.async_call
                              if async_ else self._client.sync_call)
 
+        LOG.debug(
+            "Sending an action to executor [action_ex_id=%s, action_cls=%s]",
+            action_ex_id, action_cls_str
+        )
+
         return rpc_client_method(auth_ctx.ctx(), 'run_action', **rpc_kwargs)
 
 
@@ -392,26 +397,29 @@ class EventEngineClient(evt_eng.EventEngine):
         self._client = base.get_rpc_client_driver()(rpc_conf_dict)
 
     def create_event_trigger(self, trigger, events):
-        return self._client.sync_call(
+        return self._client.async_call(
             auth_ctx.ctx(),
             'create_event_trigger',
             trigger=trigger,
-            events=events
+            events=events,
+            fanout=True,
         )
 
     def delete_event_trigger(self, trigger, events):
-        return self._client.sync_call(
+        return self._client.async_call(
             auth_ctx.ctx(),
             'delete_event_trigger',
             trigger=trigger,
-            events=events
+            events=events,
+            fanout=True,
         )
 
     def update_event_trigger(self, trigger):
-        return self._client.sync_call(
+        return self._client.async_call(
             auth_ctx.ctx(),
             'update_event_trigger',
             trigger=trigger,
+            fanout=True,
         )
 
 
