@@ -11,6 +11,7 @@
 #    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
+import yaml
 
 from mistral.db.v2 import api as db_api
 from mistral import exceptions as exc
@@ -25,7 +26,8 @@ LOG = logging.getLogger(__name__)
 
 
 def register_standard_workflows(run_in_tx=True):
-    LOG.debug("Registering standard workflows")
+    LOG.debug("Registering standard workflows...")
+
     workflow_paths = utils.get_file_list(STD_WF_PATH)
 
     for wf_path in workflow_paths:
@@ -45,16 +47,22 @@ def _clear_system_workflow_db():
 
 
 def sync_db():
-    LOG.debug("Syncing db")
+    LOG.debug("Syncing db...")
+
     with db_api.transaction():
         _clear_system_workflow_db()
+
         register_standard_workflows(run_in_tx=False)
 
 
 def create_workflows(definition, scope='private', is_system=False,
                      run_in_tx=True, namespace=''):
-    LOG.debug("Creating workflows")
-    wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(definition)
+    LOG.debug("Creating workflows...")
+
+    wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(
+        definition,
+        validate=True
+    )
     db_wfs = []
 
     if run_in_tx:
@@ -82,11 +90,16 @@ def create_workflows(definition, scope='private', is_system=False,
 
 def _append_all_workflows(definition, is_system, scope, namespace,
                           wf_list_spec, db_wfs):
-    for wf_spec in wf_list_spec.get_workflows():
+    wfs = wf_list_spec.get_workflows()
+    wfs_yaml = yaml.load(definition)
+    for wf_spec in wfs:
         db_wfs.append(
             _create_workflow(
                 wf_spec,
-                definition,
+                _cut_wf_definition_from_all(
+                    wfs_yaml,
+                    wf_spec.get_name()
+                ),
                 scope,
                 namespace,
                 is_system
@@ -98,7 +111,9 @@ def update_workflows(definition, scope='private', identifier=None,
                      namespace=''):
     LOG.debug("Updating workflows")
 
-    wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(definition)
+    wf_list_spec = spec_parser.get_workflow_list_spec_from_yaml(
+        definition, validate=True
+    )
     wfs = wf_list_spec.get_workflows()
 
     if identifier and len(wfs) > 1:
@@ -111,10 +126,14 @@ def update_workflows(definition, scope='private', identifier=None,
     db_wfs = []
 
     with db_api.transaction():
-        for wf_spec in wf_list_spec.get_workflows():
+        wfs_yaml = yaml.load(definition)
+        for wf_spec in wfs:
             db_wfs.append(_update_workflow(
                 wf_spec,
-                definition,
+                _cut_wf_definition_from_all(
+                    wfs_yaml,
+                    wf_spec.get_name()
+                ),
                 scope,
                 namespace=namespace,
                 identifier=identifier
@@ -165,6 +184,12 @@ def _update_workflow(wf_spec, definition, scope, identifier=None,
 
     return db_api.update_workflow_definition(
         identifier if identifier else values['name'],
-        values,
-        namespace=namespace
+        values
     )
+
+
+def _cut_wf_definition_from_all(wfs_yaml, wf_name):
+    return yaml.dump({
+        'version': wfs_yaml['version'],
+        wf_name: wfs_yaml[wf_name]
+    })
