@@ -532,6 +532,11 @@ class DefaultEngineTest(base.DbTestCase):
             ml_actions.Result(data='Hi')
         )
 
+        self._await(
+            lambda:
+                db_api.get_workflow_execution(wf_ex.id).state == states.SUCCESS
+        )
+
         with db_api.transaction():
             wf_ex = db_api.get_workflow_execution(wf_ex.id)
 
@@ -539,10 +544,7 @@ class DefaultEngineTest(base.DbTestCase):
 
             task_execs = wf_ex.task_executions
 
-        # Workflow completion check is done separate with scheduler
-        # but scheduler doesn't start in this test (in fact, it's just
-        # a DB test)so the workflow is expected to be in running state.
-        self.assertEqual(states.RUNNING, wf_ex.state)
+        self.assertEqual(states.SUCCESS, wf_ex.state)
 
         self.assertIsInstance(task2_action_ex, models.ActionExecution)
         self.assertEqual('std.echo', task2_action_ex.name)
@@ -625,6 +627,41 @@ class DefaultEngineTest(base.DbTestCase):
         # TODO(akhmerov): Implement.
         pass
 
+    def test_report_running_actions(self):
+        wf_input = {'param1': 'Hey', 'param2': 'Hi'}
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow(
+            'wb.wf',
+            '',
+            wf_input=wf_input,
+            description='my execution',
+            task_name='task2'
+        )
+
+        with db_api.transaction():
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_execs = wf_ex.task_executions
+
+        self.assertEqual(1, len(task_execs))
+
+        task_ex = task_execs[0]
+
+        action_execs = db_api.get_action_executions(
+            task_execution_id=task_ex.id
+        )
+
+        task_action_ex = action_execs[0]
+
+        self.engine.report_running_actions([])
+        self.engine.report_running_actions([None, None])
+        self.engine.report_running_actions([None, task_action_ex.id])
+
+        task_action_ex = db_api.get_action_execution(task_action_ex.id)
+
+        self.assertIsNotNone(task_action_ex.last_heartbeat)
+
 
 class DefaultEngineWithTransportTest(eng_test_base.EngineTestCase):
     def test_engine_client_remote_error(self):
