@@ -13,6 +13,8 @@
 #    See the License for the specific language governing permissions and
 #    limitations under the License.
 
+import six
+
 from oslo_config import cfg
 
 from mistral.db.v2 import api as db_api
@@ -753,7 +755,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         calls = db_api.get_delayed_calls()
 
-        mtd_name = 'mistral.engine.workflow_handler._check_and_complete'
+        mtd_name = 'mistral.engine.workflow_handler._check_and_fix_integrity'
 
         self._assert_single_item(calls, target_method_name=mtd_name)
 
@@ -780,7 +782,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         calls = db_api.get_delayed_calls()
 
-        mtd_name = 'mistral.engine.workflow_handler._check_and_complete'
+        mtd_name = 'mistral.engine.workflow_handler._check_and_fix_integrity'
 
         self._assert_single_item(calls, target_method_name=mtd_name)
 
@@ -1044,7 +1046,7 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
         """
 
         # Generate the workflow text.
-        task_cnt = 200
+        task_cnt = 50
 
         on_success_list_str = ''
 
@@ -1090,3 +1092,44 @@ class DirectWorkflowEngineTest(base.EngineTestCase):
 
         self._assert_single_item(task_execs, name='task0')
         self._assert_single_item(task_execs, name='task{}'.format(task_cnt))
+
+    def test_action_error_with_array_result(self):
+        wf_text = """---
+        version: '2.0'
+
+        wf:
+          tasks:
+            task1:
+              action: std.fail error_data=[1,2,3]
+        """
+
+        wf_service.create_workflows(wf_text)
+
+        # Start workflow.
+        wf_ex = self.engine.start_workflow('wf')
+
+        self.await_workflow_error(wf_ex.id)
+
+        with db_api.transaction():
+            # Note: We need to reread execution to access related tasks.
+            wf_ex = db_api.get_workflow_execution(wf_ex.id)
+
+            task_ex = self._assert_single_item(
+                wf_ex.task_executions,
+                name='task1',
+                state=states.ERROR
+            )
+
+            a_ex = self._assert_single_item(
+                task_ex.action_executions,
+                name='std.fail'
+            )
+
+            self.assertIsInstance(a_ex.output.get('result'), list)
+
+            # NOTE(rakhmerov): This was previously failing but only Python 2.7
+            # probably because SQLAlchemy works differently on different
+            # versions of Python. On Python 3 this field's value was always
+            # converted into a string no matter what we tried to assign. But
+            # that didn't happen on Python 2.7 which caused an SQL exception.
+            self.assertIsInstance(task_ex.state_info, six.string_types)
