@@ -276,63 +276,41 @@ class TaskSpecValidation(v2_base.WorkflowSpecValidationTestCase):
 
     def test_direct_transition(self):
         tests = [
-            ({'on-success': ['email']}, False),
-            ({'on-success': [{'email': '<% 1 %>'}]}, False),
-            ({'on-success': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-success': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-success': [{'email': '<% * %>'}]}, True),
-            ({'on-success': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-success': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-success': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-success': [{'email': '{{ * }}'}]}, True),
-            ({'on-success': 'email'}, False),
-            ({'on-success': None}, True),
-            ({'on-success': ['']}, True),
-            ({'on-success': []}, True),
-            ({'on-success': ['email', 'email']}, True),
-            ({'on-success': ['email', 12345]}, True),
-            ({'on-error': ['email']}, False),
-            ({'on-error': [{'email': '<% 1 %>'}]}, False),
-            ({'on-error': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-error': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-error': [{'email': '<% * %>'}]}, True),
-            ({'on-error': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-error': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-error': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-error': [{'email': '{{ * }}'}]}, True),
-            ({'on-error': 'email'}, False),
-            ({'on-error': None}, True),
-            ({'on-error': ['']}, True),
-            ({'on-error': []}, True),
-            ({'on-error': ['email', 'email']}, True),
-            ({'on-error': ['email', 12345]}, True),
-            ({'on-complete': ['email']}, False),
-            ({'on-complete': [{'email': '<% 1 %>'}]}, False),
-            ({'on-complete': [{'email': '<% 1 %>'}, 'echo']}, False),
-            ({'on-complete': [{'email': '<% $.v1 in $.v2 %>'}]}, False),
-            ({'on-complete': [{'email': '<% * %>'}]}, True),
-            ({'on-complete': [{'email': '{{ 1 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ 1 }}'}, 'echo']}, False),
-            ({'on-complete': [{'email': '{{ _.v1 in _.v2 }}'}]}, False),
-            ({'on-complete': [{'email': '{{ * }}'}]}, True),
-            ({'on-complete': 'email'}, False),
-            ({'on-complete': None}, True),
-            ({'on-complete': ['']}, True),
-            ({'on-complete': []}, True),
-            ({'on-complete': ['email', 'email']}, True),
-            ({'on-complete': ['email', 12345]}, True)
+            (['email'], False),
+            (['email%'], True),
+            ([{'email': '<% 1 %>'}], False),
+            ([{'email': '<% 1 %>'}, {'email': '<% 1 %>'}], True),
+            ([{'email': '<% 1 %>', 'more_email': '<% 2 %>'}], True),
+            (['email'], False),
+            ([{'email': '<% 1 %>'}, 'echo'], False),
+            ([{'email': '<% $.v1 in $.v2 %>'}], False),
+            ([{'email': '<% * %>'}], True),
+            ([{'email': '{{ 1 }}'}], False),
+            ([{'email': '{{ 1 }}'}, 'echo'], False),
+            ([{'email': '{{ _.v1 in _.v2 }}'}], False),
+            ([{'email': '{{ * }}'}], True),
+            ('email', False),
+            ('fail msg="<% task().result %>"', False),
+            ('fail(msg=<% task() %>)', False),
+            (None, True),
+            ([''], True),
+            ([], True),
+            (['email', 'email'], True),
+            (['email', 12345], True)
         ]
 
-        for transition, expect_error in tests:
-            overlay = {'test': {'tasks': {}}}
+        for on_clause_key in ['on-error', 'on-success', 'on-complete']:
+            for on_clause_value, expect_error in tests:
+                overlay = {'test': {'tasks': {}}}
 
-            utils.merge_dicts(overlay['test']['tasks'], {'get': transition})
+                utils.merge_dicts(overlay['test']['tasks'],
+                                  {'get': {on_clause_key: on_clause_value}})
 
-            self._parse_dsl_spec(
-                add_tasks=True,
-                changes=overlay,
-                expect_error=expect_error
-            )
+                self._parse_dsl_spec(
+                    add_tasks=True,
+                    changes=overlay,
+                    expect_error=expect_error
+                )
 
     def test_direct_transition_advanced_schema(self):
         tests = [
@@ -431,6 +409,9 @@ class TaskSpecValidation(v2_base.WorkflowSpecValidationTestCase):
             ({'on-success': {'next': 'email'}}, False),
             ({'on-success': {'next': ['email']}}, False),
             ({'on-success': {'next': [{'email': 'email'}]}}, True),
+            ({'on-success': {'next': [{'email': 'email',
+                                       'more_email': 'more_email'}]}}, True),
+            ({'on-success': {'next': {'email': 'email'}}}, True),
             ({'on-error': {'publish': {'var1': 1234}}}, True),
             ({'on-error': {'publish': {'branch': {'var1': 1234}}}}, False),
             (
@@ -526,6 +507,9 @@ class TaskSpecValidation(v2_base.WorkflowSpecValidationTestCase):
             ({'on-error': {'next': 'email'}}, False),
             ({'on-error': {'next': ['email']}}, False),
             ({'on-error': {'next': [{'email': 'email'}]}}, True),
+            ({'on-error': {'next': [{'email': 'email',
+                                     'more_email': 'more_email'}]}}, True),
+            ({'on-error': {'next': {'email': 'email'}}}, True),
             ({'on-complete': {'publish': {'var1': 1234}}}, True),
             ({'on-complete': {'publish': {'branch': {'var1': 1234}}}}, False),
             (
@@ -620,7 +604,10 @@ class TaskSpecValidation(v2_base.WorkflowSpecValidationTestCase):
             ({'on-complete': {'next': [{'email': '<% $.v1 %>'}]}}, False),
             ({'on-complete': {'next': 'email'}}, False),
             ({'on-complete': {'next': ['email']}}, False),
-            ({'on-complete': {'next': [{'email': 'email'}]}}, True)
+            ({'on-complete': {'next': [{'email': 'email'}]}}, True),
+            ({'on-complete': {'next': [{'email': 'email',
+                                        'more_email': 'more_email'}]}}, True),
+            ({'on-complete': {'next': {'email': 'email'}}}, True)
         ]
 
         for transition, expect_error in tests:
@@ -713,3 +700,24 @@ class TaskSpecValidation(v2_base.WorkflowSpecValidationTestCase):
                 changes=overlay,
                 expect_error=expect_error
             )
+
+    def test_safe_rerurn(self):
+        tests = [
+            ({'safe-rerun': True}, False),
+            ({'safe-rerun': False}, False),
+            ({'safe-rerun': '<% false %>'}, False),
+            ({'safe-rerun': '<% true %>'}, False),
+            ({'safe-rerun': '<% * %>'}, True),
+            ({'safe-rerun': None}, True)
+        ]
+
+        for default, expect_error in tests:
+            overlay = {'test': {'task-defaults': {}}}
+
+            utils.merge_dicts(overlay['test']['task-defaults'], default)
+
+            self._parse_dsl_spec(
+                add_tasks=True,
+                changes=overlay,
+                expect_error=expect_error
+            )
