@@ -22,6 +22,7 @@ import os
 from os import path
 import shutil
 import socket
+import string
 import sys
 import tempfile
 import threading
@@ -36,11 +37,14 @@ import pkg_resources as pkg
 import random
 
 from mistral import exceptions as exc
-from mistral import version
 
 
 # Thread local storage.
 _th_loc_storage = threading.local()
+
+# TODO(rakhmerov): these two constants are misplaced. Utility methods
+# should not be Mistral specific. They should be generic enough so to
+# be moved to any other project w/o changes.
 ACTION_TASK_TYPE = 'ACTION'
 WORKFLOW_TASK_TYPE = 'WORKFLOW'
 
@@ -176,10 +180,7 @@ def update_dict(left, right):
 
 
 def get_file_list(directory):
-    base_path = pkg.resource_filename(
-        version.version_info.package,
-        directory
-    )
+    base_path = pkg.resource_filename("mistral", directory)
 
     return [path.join(base_path, f) for f in os.listdir(base_path)
             if path.isfile(path.join(base_path, f))]
@@ -226,7 +227,7 @@ def cut_dict(d, length=100):
         if is_str:
             new_len += 2    # Account for the quotation marks
 
-        if new_len + len(res) >= length:
+        if 0 <= length <= new_len + len(res):
             res += "'%s" % k if is_str else k
             break
         else:
@@ -242,7 +243,7 @@ def cut_dict(d, length=100):
         if is_str:
             new_len += 2
 
-        if new_len + len(res) >= length:
+        if 0 <= length <= new_len + len(res):
             res += "'%s" % v if is_str else v
             break
         else:
@@ -252,8 +253,9 @@ def cut_dict(d, length=100):
 
         idx += 1
 
-    if len(res) >= length and res[length - 1] is not '}':
+    if 0 <= length <= len(res) and res[length - 1] is not '}':
         res = res[:length - 3] + '...'
+
     return res
 
 
@@ -273,19 +275,21 @@ def cut_list(l, length=100):
         if is_str:
             new_len += 2
 
-        if new_len >= length:
-            res += "'%s..." % s[:length - new_len] if is_str else "%s..." % s
-
+        if 0 <= length <= new_len:
+            res += "'%s" % s if is_str else s
             break
         else:
             res += "'%s'" % s if is_str else s
-            res += ', ' if idx < len(l) - 1 else ']'
+        res += ', ' if idx < len(l) - 1 else ']'
+
+    if 0 <= length <= len(res) and res[length - 1] is not ']':
+        res = res[:length - 3] + '...'
 
     return res
 
 
 def cut_string(s, length=100):
-    if len(s) > length:
+    if 0 <= length < len(s):
         return "%s..." % s[:length]
 
     return s
@@ -305,9 +309,6 @@ def cut(data, length=100):
 
 
 def cut_by_kb(data, kilobytes):
-    if kilobytes <= 0:
-        return cut(data)
-
     length = get_number_of_chars_from_kilobytes(kilobytes)
     return cut(data, length)
 
@@ -494,7 +495,12 @@ def generate_key_pair(key_length=2048):
 def utc_now_sec():
     """Returns current time and drops microseconds."""
 
-    return timeutils.utcnow().replace(microsecond=0)
+    return drop_microseconds(timeutils.utcnow())
+
+
+def drop_microseconds(date):
+    """Drops microseconds and returns date."""
+    return date.replace(microsecond=0)
 
 
 def datetime_to_str(val, sep=' '):
@@ -524,3 +530,15 @@ def datetime_to_str_in_dict(d, key, sep=' '):
 
     if val is not None:
         d[key] = datetime_to_str(d[key], sep=sep)
+
+
+def generate_string(length):
+    """Returns random string.
+
+    :param length: the length of returned string
+    """
+
+    return ''.join(random.choice(
+        string.ascii_uppercase + string.digits)
+        for _ in range(length)
+    )
